//GTT Screen and Region Tracking Demo code
//Arduino Uno with Matrix Orbital GTT43A
//Created By Daniel Divino, August 11th, 2016
//support@matrixorbital.ca
//www.matrixorbital.ca/appnotes
//www.lcdforums.ca

#include <Wire.h>
#include <stdlib.h>

//global variables
#define gttAddress 0x28 //Define default 8bit I2C address of 0x50 >> 1 for 7bit Arduino

char oldScreen = 0;
char productSelected = 0;
void setup() {
  Wire.begin();
  Serial.begin(9600);
  delay(500);
  Serial.println("Screen Tracking Ready");
}

void loop() {  
  int region;
  char screen;
  int data = i2cRead();//Read any data 
  switch (data) { //Switch case to determine what to do when the arduino receieves a return message from the GTT
    case 252: { 
      data = i2cRead(); //When return prefix 252 from the GTT is received, read the next byte of data
      switch (data) {
        case 135: { //135 = region/button was pressed
          region = ReadRegion(); //read the rest of the data. Return what region/button has been pressed
          Serial.print("Region = "); 
          Serial.println(region); //Display what region was pressed on the serial monitor.
          if(region == 4){ //If region 4 was pressed, display the selected product's information.
            Serial.println("Product Selected"); 
            productSelected = oldScreen;
            updateProductScreen();                                                      
          }
          if(region == 5){ //If product information is displayed, and region 5 (home button) was pressed, return to the home menu.
              returnToMenu();            
          }
        }
        break;
        case 255: { //255 = the screen has changed/transitioned to a new screen.
          screen = ReadScreen(); //read what the current screen is.                                         
          if (oldScreen != screen) { //if the new screen number != old screen number, the GTT has transitioned, and is displaying a new screen.
            oldScreen = screen;
            Serial.println("\nTranistion to new screen!");
            Serial.print("New Screen = ");
            Serial.println(screen);
          }
          else { //If the new screen number is the same as the old screen number, the GTT is still on the same screen. Display what screen is being displayed
            Serial.print("\nCurrent Screen = ");
            Serial.println(screen);
          }             
        }
        break;
      }
    }
    break;
  }
}

void updateProductScreen(){
  setManualUpdate(1);  //Enable manual update on the display  
  byte gttInfoLengths[] = {6,10,9,16};    
  switch(oldScreen) {
    case 49: {
      char* gtt35AInfo[] = {"GTT35A","3.5 inches","320 x 240","Images\\GTT35.jpg"}; 
      for (byte i = 0; i<3; i++){ //Update the product title,  product size, and product resolution.
        updateLabel(i, 0, gtt35AInfo[i], gttInfoLengths[i]);   
      }                   
      loadBitmap(10,gtt35AInfo[3], gttInfoLengths[3]); //Load and display product image
      displayBitmap(10,16,41);      
      UpdateText(0); //Update the selected product information
    }
    break;
    case 50: {
      char* gtt38AInfo[] = {"GTT38A","3.8 inches","480 x 116","Images\\GTT38.jpg"};      
      for (byte i = 0; i<3; i++){ //Update the product title,  product size, and product resolution.
        updateLabel(i, 0, gtt38AInfo[i], gttInfoLengths[i]);   
      }          
      loadBitmap(10,gtt38AInfo[3], gttInfoLengths[3]); //Load and display product image
      displayBitmap(10,4,107);       
      UpdateText(1); //Update the selected product information
    }          
    break;
    case 51: {
      char* gtt43AInfo[] = {"GTT43A","4.3 inches","480 x 272","Images\\GTT43.jpg"};
      for (byte i = 0; i<3; i++){ //Update the product title,  product size, and product resolution.
        updateLabel(i, 0, gtt43AInfo[i], gttInfoLengths[i]);   
      }    
      loadBitmap(10,gtt43AInfo[3], gttInfoLengths[3]); //Load and display product image
      displayBitmap(10,4,67); 
      UpdateText(2); //Update the selected product information
    }
    break;
    case 52: {
      char* gtt50AInfo[] = {"GTT50A","5.0 inches","800 x 480","Images\\GTT50.jpg"};
      for (byte i = 0; i<3; i++){ //Update the product title,  product size, and product resolution.
        updateLabel(i, 0, gtt50AInfo[i], gttInfoLengths[i]);   
      }          
      loadBitmap(10,gtt50AInfo[3], gttInfoLengths[3]); //Load and display product image  
      displayBitmap(10,4,66);              
      UpdateText(3); //Update the selected product information
    }
    break;
    case 53: {
      char* gtt70AInfo[] = {"GTT70A","7.0 inches","800 x 480","Images\\GTT70.jpg"};
      for (byte i = 0; i<3; i++){ //Update the product title,  product size, and product resolution.
        updateLabel(i, 0, gtt70AInfo[i], gttInfoLengths[i]);   
      }    
      loadBitmap(10,gtt70AInfo[3], gttInfoLengths[3]); //Load and display product image
      displayBitmap(10,4,71);   
      UpdateText(4); //Update the selected product information           
    }
    break;    
  }
  manualUpdate(); //Manually update the screen with the new product descriptions
  setManualUpdate(0); //disable manual updates
}

void UpdateText(int message){  
  int gttDescriptionLengths[] = {179, 163, 189,176,89};      
  char* gttDescriptions[] = { //Array of product descriptions                  
    "The Matrix Orbital GTT35A is\na full color, 3.5 inch TFT\ndisplay with an integrated\ntouch screen. It is ideal for\nprojects that require a\npowerful display,in a smaller\nform factor.",
    "The GTT38A from Matrix \nOrbital is a 1U high, full color,\n3.8 inch TFT touch screen\ndisplay. Its thin profile, and\nheight makes it perfect for use\non server racks.", 
    "Matrix Orbital's GTT43A is a\nfull color 4.3 inch TFT touch\nscreen display with a 480x272\nresolution. The GTT43A is\nideal for projects that focus on displaying information to it's\noperators.",
    "The full color, 5 inch wide,\n800x480 display found on the\nMatrix Orbital GTT50A makes it perfect for projects that\ndisplay plenty of information\nwith great detail to its\nusers.",
    "Matrix Orbital's GTT70A is a\nfull color, 7.0 inch TFT\ndisplay, with a 800x480\nresolution."
  };   
  i2cUpdateTextWindow(gttDescriptions[message],gttDescriptionLengths[message]);
  
}

void returnToMenu(){
  Serial.println("Return to menu"); //Let user know that the program will be returning to the main menu  
  byte scriptFileLength = 31;         
  switch(productSelected){
    case 49:{
      char* scriptFileName = {"Screen Tracking\\GTT35\\GTT35.bin"};      
      runScriptFile(scriptFileName,scriptFileLength); //Return to the main menu
    }
    break;
    case 50:{
      char* scriptFileName = "Screen Tracking\\GTT38\\GTT38.bin";
      runScriptFile(scriptFileName,scriptFileLength); //Return to the main menu
    }
    break;
    case 51:{
      char* scriptFileName = "Screen Tracking\\GTT43\\GTT43.bin";
      runScriptFile(scriptFileName,scriptFileLength); //Return to the main menu
    }
    break;
    case 52:{
      char* scriptFileName = "Screen Tracking\\GTT50\\GTT50.bin";              
      runScriptFile(scriptFileName,scriptFileLength); //Return to the main menu    
    }
    break;
    case 53:{      
      char* scriptFileName = "Screen Tracking\\GTT70\\GTT70.bin";              
      runScriptFile(scriptFileName,scriptFileLength); //Return to the main menu      
    }
    break;   
    default:{
      char* scriptFileName = "Screen Tracking\\GTT43\\GTT43.bin";              
      runScriptFile(scriptFileName,scriptFileLength); //Return to the main menu      
    }
    break;
  }   
}

int ReadRegion() { //Determine which region has been pressed
  int returnLength = i2cRead() << 16 | i2cRead();  //Read mesage length
  int event = i2cRead();  //Read event type
  int region = i2cRead();  //Read event type
  return region;
}

char ReadScreen() { //Determine what screen you are currently on
  char message;
  int returnLength1 = i2cRead() << 16 | i2cRead();  //Read mesage length
  message = i2cRead();
  return message;
}

//GTT Protocol Commands 
void setManualUpdate(byte setting){
  char command[] = { 254, 90, setting};
  i2cWrite(command, sizeof(command));
}

void manualUpdate(){
  char command[] = { 254, 91};
  i2cWrite(command, sizeof(command));
}

void loadBitmap(byte index, char* message, byte messageLength){
  char command[] = { 254, 95, index};
  i2cWrite(command, sizeof(command));
  i2cWrite(message, messageLength);
  char terminator[] = {0};
  i2cWrite(terminator, 1);
}

void displayBitmap(byte index, short x, short y){
  char command[] = { 254, 97, index,(byte)(x >>8),(byte)(x),(byte)(y>>8),(byte)(y)};
  i2cWrite(command, sizeof(command));
}

void runScriptFile(char* message, byte messageLength){
  char command[] = { 254, 93};
  i2cWrite(command, sizeof(command));
  i2cWrite(message, messageLength);
  char terminator[] = {0};
  i2cWrite(terminator, 1);
}

void clearABuffer(byte type, byte index){
  char command[] = { 254, 208, type, index };
  i2cWrite(command, sizeof(command));
}

void updateLabel(byte index, byte format, char* message, byte messageLength){
  char command[] = { 254, 17, index, format };
  i2cWrite(command, sizeof(command));
  i2cWrite(message, messageLength);
  char terminator[] = { 0 };
  i2cWrite(terminator, 1);
}

//I2C communication commands
byte i2cRead() {
  do {
    Wire.requestFrom(gttAddress, 1);
  } while (Wire.available() < 1);
  return (Wire.read());
}

void i2cWrite(char* data, byte data_length) {
  Wire.beginTransmission(gttAddress);
  for (int i = 0; i < data_length; i++) {
    Wire.write(data[i]);
  }
  Wire.endTransmission();
}

void i2cUpdateTextWindow(char* message, byte data_length){  
  int iterations = data_length/32;
  int divisor = data_length%32;
  int i = 0, j = 0, k=0;  
  for (j = 0; j<(iterations); j++){
    Wire.beginTransmission(gttAddress);
    Wire.write(message+i,32);
    Wire.endTransmission();  
    i +=32;
  }      
  if (j == (iterations)){        
    Wire.beginTransmission(gttAddress);            
    Wire.write(message+i,divisor);
    Wire.endTransmission();      
  }
}

